/*
 * QLogic iSCSI HBA Driver
 * Copyright (c)  2003-2011 QLogic Corporation
 *
 * See LICENSE.qla4xxx for copyright and licensing details.
 *
 * PCI searching functions pci_get_domain_bus_and_slot & pci_channel_offline
 * Copyright (C) 1993 -- 1997 Drew Eckhardt, Frederic Potter,
 *                                      David Mosberger-Tang
 * Copyright (C) 1997 -- 2000 Martin Mares <mj@ucw.cz>
 * Copyright (C) 2003 -- 2004 Greg Kroah-Hartman <greg@kroah.com>.
 */

#ifndef __QLA_KCOMPAT_H
#define __QLA_KCOMPAT_H
#include <linux/version.h>
#include <linux/pci.h>

#ifndef DID_TRANSPORT_DISRUPTED
#define DID_TRANSPORT_DISRUPTED DID_BUS_BUSY
#endif

#ifndef list_first_entry
#define list_first_entry(ptr, type, member) \
	list_entry((ptr)->next, type, member)
#endif

#if (RHEL_MINOR < 3) /* For RHEL 5.2 compatibility */
static inline int pci_channel_offline(struct pci_dev *pdev)
{
	return pdev->error_state != pci_channel_io_normal;
}
#endif

#ifdef QL4_OVM22
#include <scsi/scsi_transport_iscsi2.h>
#else
#if (RHEL_MINOR < 4) /* For RHEL 5.3 and below compatibility */
#include <scsi/scsi_transport_iscsi.h>
#else
#include <scsi/scsi_transport_iscsi2.h>
#endif

#if (RHEL_MINOR < 5) /* For RHEL 5.4 and below compatibility */
static inline struct pci_dev *pci_get_domain_bus_and_slot(int domain, unsigned int bus,
                                            unsigned int devfn)
{
        struct pci_dev *dev = NULL;

        while ((dev = pci_get_device(PCI_ANY_ID, PCI_ANY_ID, dev)) != NULL) {
                if (pci_domain_nr(dev->bus) == domain &&
                    (dev->bus->number == bus && dev->devfn == devfn))
                        return dev;
        }
        return NULL;
}
#endif

#if (RHEL_MINOR < 4) /* For RHEL 5.3 and below compatibility */
static inline void iscsi2_unblock_session(struct iscsi_cls_session *session)
{
	iscsi_unblock_session(session);
}

static inline void iscsi2_block_session(struct iscsi_cls_session *session)
{
	iscsi_block_session(session);
}

static inline void iscsi2_destroy_conn(struct iscsi_cls_conn *conn)
{
	iscsi_if_destroy_session_done(conn);
	iscsi_destroy_conn(conn);
}

static inline void iscsi2_remove_session(struct iscsi_cls_session *session)
{
	iscsi_remove_session(session);
}

static inline void iscsi2_free_session(struct iscsi_cls_session *session)
{
	iscsi_free_session(session);
}

static inline int iscsi2_add_session(struct iscsi_cls_session *session, unsigned int target_id)
{
	return iscsi_add_session(session, target_id);
}

static inline struct iscsi_cls_conn *
iscsi2_create_conn(struct iscsi_cls_session *session, int dd_size, uint32_t cid)
{
	return iscsi_create_conn(session, cid);
}

static inline struct iscsi_cls_session *
iscsi2_alloc_session(struct Scsi_Host *shost, struct iscsi_transport *transport,
			int dd_size)
{
	return iscsi_alloc_session(shost, transport);
}

static inline struct scsi_transport_template *
iscsi2_register_transport(struct iscsi_transport *tt)
{
	return iscsi_register_transport(tt);
}

static inline int iscsi2_unregister_transport(struct iscsi_transport *tt)
{
	return iscsi_unregister_transport(tt);
}

static inline int iscsi2_session_chkready(struct iscsi_cls_session *sess)
{
	/*
	 * iscsi_session_chkready function is not defined in
	 * RHEL 53 kernel.
	 */
	return 0;
}

#endif
#endif

#if defined(CONFIG_PCIEAER)
#include <linux/aer.h>
#else
/* AER releated */
static inline int pci_enable_pcie_error_reporting(struct pci_dev *dev)
{
	return -EINVAL;
}
static inline int pci_disable_pcie_error_reporting(struct pci_dev *dev)
{
	return -EINVAL;
}
static inline int pci_cleanup_aer_uncorrect_error_status(struct pci_dev *dev)
{
	return -EINVAL;
}
#endif

#endif

